@@ -76,7 +76,6 @@ typedef enum /*< enum >*/
 
 /**
  * ThunarFileThumbState:
- * @THUNAR_FILE_THUMB_STATE_MASK    : the mask to extract the thumbnail state.
  * @THUNAR_FILE_THUMB_STATE_UNKNOWN : unknown whether there's a thumbnail.
  * @THUNAR_FILE_THUMB_STATE_NONE    : no thumbnail is available.
  * @THUNAR_FILE_THUMB_STATE_READY   : a thumbnail is available.
@@ -86,17 +85,18 @@ typedef enum /*< enum >*/
  **/
 typedef enum /*< flags >*/
 {
-  THUNAR_FILE_THUMB_STATE_MASK    = 0x03,
-  THUNAR_FILE_THUMB_STATE_UNKNOWN = 0x00,
-  THUNAR_FILE_THUMB_STATE_NONE    = 0x01,
-  THUNAR_FILE_THUMB_STATE_READY   = 0x02,
-  THUNAR_FILE_THUMB_STATE_LOADING = 0x03,
+  THUNAR_FILE_THUMB_STATE_UNKNOWN = 0,
+  THUNAR_FILE_THUMB_STATE_NONE    = 1,
+  THUNAR_FILE_THUMB_STATE_READY   = 2,
+  THUNAR_FILE_THUMB_STATE_LOADING = 3,
 } ThunarFileThumbState;
 
+
+
 #define THUNAR_FILE_EMBLEM_NAME_SYMBOLIC_LINK "emblem-symbolic-link"
-#define THUNAR_FILE_EMBLEM_NAME_CANT_READ "emblem-noread"
-#define THUNAR_FILE_EMBLEM_NAME_CANT_WRITE "emblem-nowrite"
-#define THUNAR_FILE_EMBLEM_NAME_DESKTOP "emblem-desktop"
+#define THUNAR_FILE_EMBLEM_NAME_CANT_READ     "emblem-noread"
+#define THUNAR_FILE_EMBLEM_NAME_CANT_WRITE    "emblem-nowrite"
+#define THUNAR_FILE_EMBLEM_NAME_DESKTOP       "emblem-desktop"
 
 
 
@@ -118,12 +118,15 @@ GType             thunar_file_get_type             (void) G_GNUC_CONST;
 
 ThunarFile       *thunar_file_get                  (GFile                  *file,
                                                     GError                **error);
+ThunarFile       *thunar_file_get_with_info        (GFile                  *file,
+                                                    GFileInfo              *info,
+                                                    gboolean                not_mounted);
 ThunarFile       *thunar_file_get_for_uri          (const gchar            *uri,
                                                     GError                **error);
-void              thunar_file_get_async            (GFile                 *location,
-                                                    GCancellable          *cancellable,
-                                                    ThunarFileGetFunc      func,
-                                                    gpointer               user_data);
+void              thunar_file_get_async            (GFile                  *location,
+                                                    GCancellable           *cancellable,
+                                                    ThunarFileGetFunc       func,
+                                                    gpointer                user_data);
 
 GFile            *thunar_file_get_file             (const ThunarFile       *file) G_GNUC_PURE;
 
@@ -179,9 +182,6 @@ GAppInfo         *thunar_file_get_default_handler  (const ThunarFile       *file
 GFileType         thunar_file_get_kind             (const ThunarFile       *file) G_GNUC_PURE;
 GFile            *thunar_file_get_target_location  (const ThunarFile       *file);
 ThunarFileMode    thunar_file_get_mode             (const ThunarFile       *file);
-gboolean          thunar_file_get_free_space       (const ThunarFile       *file, 
-                                                    guint64                *fs_free_return,
-                                                    guint64                *fs_size_return);
 gboolean          thunar_file_is_mounted           (const ThunarFile       *file);
 gboolean          thunar_file_exists               (const ThunarFile       *file);
 gboolean          thunar_file_is_directory         (const ThunarFile       *file) G_GNUC_PURE;
