@@ -864,3 +864,21 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
 
   return icon;
 }
+
+
+
+/**
+ * thunar_icon_factory_clear_pixmap_cache:
+ * @file : a #ThunarFile.
+ *
+ * Unset the pixmap cache on a file to force a reload on the next request.
+ **/
+void
+thunar_icon_factory_clear_pixmap_cache (ThunarFile *file)
+{
+  _thunar_return_if_fail (THUNAR_IS_FILE (file));
+
+  /* unset the data */
+  if (thunar_icon_factory_store_quark != 0)
+    g_object_set_qdata (G_OBJECT (file), thunar_icon_factory_store_quark, NULL);
+}
