@@ -188,7 +188,7 @@ thunar_progress_dialog_toggled (ThunarProgressDialog *dialog,
   _thunar_return_val_if_fail (GTK_IS_STATUS_ICON (status_icon), FALSE);
 
   /* check if the window is visible and has the focus */
-  if (GTK_WIDGET_VISIBLE (GTK_WIDGET (dialog)) 
+  if (gtk_widget_get_visible (GTK_WIDGET (dialog)) 
       && gtk_window_is_active (GTK_WINDOW (dialog)))
     {
       /* remember the position of the dialog */
@@ -200,7 +200,7 @@ thunar_progress_dialog_toggled (ThunarProgressDialog *dialog,
   else
     {
       /* check if the dialog is invisible */
-      if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (dialog)))
+      if (!gtk_widget_get_visible (GTK_WIDGET (dialog)))
         {
           /* restore its previous position before presenting it */
           gtk_window_move (GTK_WINDOW (dialog), dialog->x, dialog->y);
@@ -302,11 +302,7 @@ thunar_progress_dialog_update_status_icon (ThunarProgressDialog *dialog)
                                             n_views);
 
   /* update the tooltip */
-#if GTK_CHECK_VERSION (2, 16, 0)
   gtk_status_icon_set_tooltip_text (dialog->status_icon, tooltip_text);
-#else
-  gtk_status_icon_set_tooltip (dialog->status_icon, tooltip_text);
-#endif
 
   /* free the string */
   g_free (tooltip_text);
@@ -343,7 +339,11 @@ thunar_progress_dialog_add_job (ThunarProgressDialog *dialog,
   thunar_progress_view_set_title (THUNAR_PROGRESS_VIEW (view), title);
   gtk_box_pack_start (GTK_BOX (dialog->content_box), view, FALSE, TRUE, 0);
   gtk_widget_show (view);
-  
+
+  /* use the first job's icon-name for the dialog */
+  if (dialog->views == NULL)
+    gtk_window_set_icon_name (GTK_WINDOW (dialog), icon_name);
+
   /* add the view to the list of known views */
   dialog->views = g_list_prepend (dialog->views, view);
 
