@@ -855,14 +855,10 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
   const gchar       *name;
   const gchar       *path;
   GVolume           *volume;
-  guint64            fs_free;
-  guint64            fs_size;
   GIcon             *gicon;
   glong              offset;
   gchar             *date;
   gchar             *display_name;
-  gchar             *fs_free_str;
-  gchar             *fs_size_str;
   gchar             *fs_string;
   gchar             *str;
   gchar             *volume_name;
@@ -1045,21 +1041,19 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
     }
 
   /* update the free space (only for folders) */
-  if (thunar_file_is_directory (file)
-      && thunar_file_get_free_space (file, &fs_free, &fs_size)
-      && fs_size > 0)
+  if (thunar_file_is_directory (file))
     {
-      fs_free_str = g_format_size (fs_free);
-      fs_size_str = g_format_size (fs_size);
-      fs_string = g_strdup_printf (_("%s of %s (%d%% used)"),
-                                   fs_free_str, fs_size_str,
-                                   (gint) ((fs_size - fs_free) * 100 / fs_size));
-      g_free (fs_free_str);
-      g_free (fs_size_str);
-
-      gtk_label_set_text (GTK_LABEL (dialog->freespace_label), fs_string);
-      gtk_widget_show (dialog->freespace_label);
-      g_free (fs_string);
+      fs_string = thunar_g_file_get_free_space_string (thunar_file_get_file (file));
+      if (fs_string != NULL)
+        {
+          gtk_label_set_text (GTK_LABEL (dialog->freespace_label), fs_string);
+          gtk_widget_show (dialog->freespace_label);
+          g_free (fs_string);
+        }
+      else
+        {
+          gtk_widget_hide (dialog->freespace_label);
+        }
     }
   else
     {
