@@ -37,16 +37,18 @@ typedef struct _ThunarRenamerModel      ThunarRenamerModel;
 
 /**
  * ThunarRenamerModelColumn:
- * @THUNAR_RENAMER_MODEL_COLUMN_CONFLICT : the column which tells whether there's a name conflict.
- * @THUNAR_RENAMER_MODEL_COLUMN_FILE     : the column with the #ThunarFile.
- * @THUNAR_RENAMER_MODEL_COLUMN_NEWNAME  : the column with the new name.
- * @THUNAR_RENAMER_MODEL_COLUMN_OLDNAME  : the column with the old name.
+ * @THUNAR_RENAMER_MODEL_COLUMN_CONFLICT        : the column which tells whether there's a name conflict.
+ * @THUNAR_RENAMER_MODEL_COLUMN_CONFLICT_WEIGHT : Use to set the text to bold in case of a conflict
+ * @THUNAR_RENAMER_MODEL_COLUMN_FILE            : the column with the #ThunarFile.
+ * @THUNAR_RENAMER_MODEL_COLUMN_NEWNAME         : the column with the new name.
+ * @THUNAR_RENAMER_MODEL_COLUMN_OLDNAME         : the column with the old name.
  *
  * The column ids provided by #ThunarRenamerModel instances.
  **/
 typedef enum
 {
   THUNAR_RENAMER_MODEL_COLUMN_CONFLICT,
+  THUNAR_RENAMER_MODEL_COLUMN_CONFLICT_WEIGHT,
   THUNAR_RENAMER_MODEL_COLUMN_FILE,
   THUNAR_RENAMER_MODEL_COLUMN_NEWNAME,
   THUNAR_RENAMER_MODEL_COLUMN_OLDNAME,
@@ -69,6 +71,8 @@ void                 thunar_renamer_model_insert         (ThunarRenamerModel *re
 void                 thunar_renamer_model_reorder        (ThunarRenamerModel *renamer_model,
                                                           GList              *tree_paths,
                                                           gint                position);
+void                 thunar_renamer_model_sort           (ThunarRenamerModel *renamer_model,
+                                                          GtkSortType         sort_order);
 void                 thunar_renamer_model_clear          (ThunarRenamerModel *renamer_model);
 void                 thunar_renamer_model_remove         (ThunarRenamerModel *renamer_model,
                                                           GtkTreePath        *path);
