@@ -119,25 +119,26 @@ struct _ThunarTextRenderer
 {
   GtkCellRenderer __parent__;
 
+  /* pango renderer properties */
   PangoLayout    *layout;
   GtkWidget      *widget;
-  gboolean        text_static;
+  guint           text_static : 1;
   gchar          *text;
   gint            char_width;
   gint            char_height;
   PangoWrapMode   wrap_mode;
   gint            wrap_width;
-  gboolean        follow_state;
+  guint           follow_state : 1;
   gint            focus_width;;
   PangoAlignment  alignment;
 
   /* underline prelited rows */
-  gboolean        follow_prelit;
+  guint           follow_prelit : 1;
 
   /* cell editing support */
   GtkWidget      *entry;
-  gboolean        entry_menu_active;
-  gint            entry_menu_popdown_timer_id;
+  guint           entry_menu_active : 1;
+  guint           entry_menu_popdown_timer_id;
 };
 
 
@@ -279,7 +280,6 @@ static void
 thunar_text_renderer_init (ThunarTextRenderer *text_renderer)
 {
   text_renderer->wrap_width = -1;
-  text_renderer->entry_menu_popdown_timer_id = -1;
   text_renderer->alignment = PANGO_ALIGN_LEFT;
 }
 
@@ -794,7 +794,7 @@ thunar_text_renderer_populate_popup (GtkEntry           *entry,
                                      GtkMenu            *menu,
                                      ThunarTextRenderer *text_renderer)
 {
-  if (G_UNLIKELY (text_renderer->entry_menu_popdown_timer_id >= 0))
+  if (G_UNLIKELY (text_renderer->entry_menu_popdown_timer_id != 0))
     g_source_remove (text_renderer->entry_menu_popdown_timer_id);
 
   text_renderer->entry_menu_active = TRUE;
@@ -810,7 +810,7 @@ thunar_text_renderer_popup_unmap (GtkMenu            *menu,
 {
   text_renderer->entry_menu_active = FALSE;
 
-  if (G_LIKELY (text_renderer->entry_menu_popdown_timer_id < 0))
+  if (G_LIKELY (text_renderer->entry_menu_popdown_timer_id == 0))
     {
       text_renderer->entry_menu_popdown_timer_id = g_timeout_add_full (G_PRIORITY_LOW, 500u, thunar_text_renderer_entry_menu_popdown_timer,
                                                                        text_renderer, thunar_text_renderer_entry_menu_popdown_timer_destroy);
@@ -840,7 +840,7 @@ thunar_text_renderer_entry_menu_popdown_timer (gpointer user_data)
 static void
 thunar_text_renderer_entry_menu_popdown_timer_destroy (gpointer user_data)
 {
-  THUNAR_TEXT_RENDERER (user_data)->entry_menu_popdown_timer_id = -1;
+  THUNAR_TEXT_RENDERER (user_data)->entry_menu_popdown_timer_id = 0;
 }
 
 
